@@ -41,11 +41,12 @@
 #include "pcap-usb-linux.h"
 #include "pcap/usb.h"
 
+#include "extract.h"
+
 #ifdef NEED_STRERROR_H
 #include "strerror.h"
 #endif
 
-#include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -136,7 +137,7 @@ static int usb_stats_linux_bin(pcap_t *, struct pcap_stat *);
 static int usb_read_linux(pcap_t *, int , pcap_handler , u_char *);
 static int usb_read_linux_bin(pcap_t *, int , pcap_handler , u_char *);
 static int usb_read_linux_mmap(pcap_t *, int , pcap_handler , u_char *);
-static int usb_inject_linux(pcap_t *, const void *, size_t);
+static int usb_inject_linux(pcap_t *, const void *, int);
 static int usb_setdirection_linux(pcap_t *, pcap_direction_t);
 static void usb_cleanup_linux_mmap(pcap_t *);
 
@@ -145,7 +146,7 @@ have_binary_usbmon(void)
 {
 	struct utsname utsname;
 	char *version_component, *endp;
-	int major, minor, subminor;
+	long major, minor, subminor;
 
 	if (uname(&utsname) == 0) {
 		/*
@@ -230,7 +231,7 @@ usb_dev_add(pcap_if_list_t *devlistp, int n, char *err_str)
 {
 	char dev_name[10];
 	char dev_descr[30];
-	pcap_snprintf(dev_name, 10, USB_IFACE"%d", n);
+	snprintf(dev_name, 10, USB_IFACE"%d", n);
 	/*
 	 * XXX - is there any notion of "up" and "running"?
 	 */
@@ -251,7 +252,7 @@ usb_dev_add(pcap_if_list_t *devlistp, int n, char *err_str)
 		 * PCAP_IF_CONNECTION_STATUS_CONNECTED or
 		 * PCAP_IF_CONNECTION_STATUS_DISCONNECTED?
 		 */
-		pcap_snprintf(dev_descr, 30, "Raw USB traffic, bus number %d", n);
+		snprintf(dev_descr, 30, "Raw USB traffic, bus number %d", n);
 		if (add_dev(devlistp, dev_name, 0, dev_descr, err_str) == NULL)
 			return -1;
 	}
@@ -363,9 +364,9 @@ usb_findalldevs(pcap_if_list_t *devlistp, char *err_str)
 				len = strlen(name);
 
 				/* if this file name does not end with a number it's not of our interest */
-				if ((len < 1) || !isdigit(name[--len]))
+				if ((len < 1) || !PCAP_ISDIGIT(name[--len]))
 					continue;
-				while (isdigit(name[--len]));
+				while (PCAP_ISDIGIT(name[--len]));
 				if (sscanf(&name[len+1], "%d", &n) != 1)
 					continue;
 
@@ -490,6 +491,10 @@ int usb_mmap(pcap_t* handle)
 #define USB_REQ_GET_DESCRIPTOR	6
 
 #define USB_DT_DEVICE		1
+#define USB_DT_CONFIG		2
+
+#define USB_DEVICE_DESCRIPTOR_SIZE	18
+#define USB_CONFIG_DESCRIPTOR_SIZE	9
 
 /* probe the descriptors of the devices attached to the bus */
 /* the descriptors will end up in the captured packet stream */
@@ -501,12 +506,14 @@ probe_devices(int bus)
 	struct usbdevfs_ctrltransfer ctrl;
 	struct dirent* data;
 	int ret = 0;
-	char buf[sizeof("/dev/bus/usb/000/") + NAME_MAX];
+	char busdevpath[sizeof("/dev/bus/usb/000/") + NAME_MAX];
 	DIR* dir;
+	uint8_t descriptor[USB_DEVICE_DESCRIPTOR_SIZE];
+	uint8_t configdesc[USB_CONFIG_DESCRIPTOR_SIZE];
 
 	/* scan usb bus directories for device nodes */
-	pcap_snprintf(buf, sizeof(buf), "/dev/bus/usb/%03d", bus);
-	dir = opendir(buf);
+	snprintf(busdevpath, sizeof(busdevpath), "/dev/bus/usb/%03d", bus);
+	dir = opendir(busdevpath);
 	if (!dir)
 		return;
 
@@ -517,9 +524,9 @@ probe_devices(int bus)
 		if (name[0] == '.')
 			continue;
 
-		pcap_snprintf(buf, sizeof(buf), "/dev/bus/usb/%03d/%s", bus, data->d_name);
+		snprintf(busdevpath, sizeof(busdevpath), "/dev/bus/usb/%03d/%s", bus, data->d_name);
 
-		fd = open(buf, O_RDWR);
+		fd = open(busdevpath, O_RDWR);
 		if (fd == -1)
 			continue;
 
@@ -532,19 +539,43 @@ probe_devices(int bus)
 		ctrl.bRequest = USB_REQ_GET_DESCRIPTOR;
 		ctrl.wValue = USB_DT_DEVICE << 8;
 		ctrl.wIndex = 0;
- 		ctrl.wLength = sizeof(buf);
+ 		ctrl.wLength = sizeof(descriptor);
 #else
 		ctrl.requesttype = USB_DIR_IN | USB_TYPE_STANDARD | USB_RECIP_DEVICE;
 		ctrl.request = USB_REQ_GET_DESCRIPTOR;
 		ctrl.value = USB_DT_DEVICE << 8;
 		ctrl.index = 0;
- 		ctrl.length = sizeof(buf);
+ 		ctrl.length = sizeof(descriptor);
 #endif
-		ctrl.data = buf;
+		ctrl.data = descriptor;
 		ctrl.timeout = CTRL_TIMEOUT;
 
 		ret = ioctl(fd, USBDEVFS_CONTROL, &ctrl);
 
+		/* Request CONFIGURATION descriptor alone to know wTotalLength */
+#ifdef HAVE_STRUCT_USBDEVFS_CTRLTRANSFER_BREQUESTTYPE
+		ctrl.wValue = USB_DT_CONFIG << 8;
+		ctrl.wLength = sizeof(configdesc);
+#else
+		ctrl.value = USB_DT_CONFIG << 8;
+		ctrl.length = sizeof(configdesc);
+#endif
+		ctrl.data = configdesc;
+		ret = ioctl(fd, USBDEVFS_CONTROL, &ctrl);
+		if (ret >= 0) {
+			uint16_t wtotallength;
+			wtotallength = EXTRACT_LE_U_2(&configdesc[2]);
+#ifdef HAVE_STRUCT_USBDEVFS_CTRLTRANSFER_BREQUESTTYPE
+			ctrl.wLength = wtotallength;
+#else
+			ctrl.length = wtotallength;
+#endif
+			ctrl.data = malloc(wtotallength);
+			if (ctrl.data) {
+				ret = ioctl(fd, USBDEVFS_CONTROL, &ctrl);
+				free(ctrl.data);
+			}
+		}
 		close(fd);
 	}
 	closedir(dir);
@@ -586,7 +617,7 @@ usb_create(const char *device, char *ebuf, int *is_ours)
 	/* OK, it's probably ours. */
 	*is_ours = 1;
 
-	p = pcap_create_common(ebuf, sizeof (struct pcap_usb_linux));
+	p = PCAP_CREATE_COMMON(ebuf, struct pcap_usb_linux);
 	if (p == NULL)
 		return (NULL);
 
@@ -627,7 +658,7 @@ usb_activate(pcap_t* handle)
 	/*get usb bus index from device name */
 	if (sscanf(handle->opt.device, USB_IFACE"%d", &handlep->bus_index) != 1)
 	{
-		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 			"Can't get USB bus index from %s", handle->opt.device);
 		return PCAP_ERROR;
 	}
@@ -638,7 +669,7 @@ usb_activate(pcap_t* handle)
 		 * We have binary-mode support.
 		 * Try to open the binary interface.
 		 */
-		pcap_snprintf(full_path, USB_LINE_LEN, LINUX_USB_MON_DEV"%d", handlep->bus_index);
+		snprintf(full_path, USB_LINE_LEN, LINUX_USB_MON_DEV"%d", handlep->bus_index);
 		handle->fd = open(full_path, O_RDONLY, 0);
 		if (handle->fd < 0)
 		{
@@ -731,7 +762,7 @@ usb_activate(pcap_t* handle)
 		 * We don't have binary mode support.
 		 * Try opening the text-mode device.
 		 */
-		pcap_snprintf(full_path, USB_LINE_LEN, USB_TEXT_DIR"/%dt", handlep->bus_index);
+		snprintf(full_path, USB_LINE_LEN, USB_TEXT_DIR"/%dt", handlep->bus_index);
 		handle->fd = open(full_path, O_RDONLY, 0);
 		if (handle->fd < 0)
 		{
@@ -741,7 +772,7 @@ usb_activate(pcap_t* handle)
 				 * Not found at the new location; try
 				 * the old location.
 				 */
-				pcap_snprintf(full_path, USB_LINE_LEN, USB_TEXT_DIR_OLD"/%dt", handlep->bus_index);
+				snprintf(full_path, USB_LINE_LEN, USB_TEXT_DIR_OLD"/%dt", handlep->bus_index);
 				handle->fd = open(full_path, O_RDONLY, 0);
 			}
 			if (handle->fd < 0) {
@@ -832,6 +863,7 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 	struct pcap_usb_linux *handlep = handle->priv;
 	unsigned timestamp;
 	int tag, cnt, ep_num, dev_addr, dummy, ret, urb_len, data_len;
+	ssize_t read_ret;
 	char etype, pipeid1, pipeid2, status[16], urb_tag, line[USB_LINE_LEN];
 	char *string = line;
 	u_char * rawdata = handle->buffer;
@@ -842,14 +874,14 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 
 	/* ignore interrupt system call errors */
 	do {
-		ret = read(handle->fd, line, USB_LINE_LEN - 1);
+		read_ret = read(handle->fd, line, USB_LINE_LEN - 1);
 		if (handle->break_loop)
 		{
 			handle->break_loop = 0;
 			return -2;
 		}
-	} while ((ret == -1) && (errno == EINTR));
-	if (ret < 0)
+	} while ((read_ret == -1) && (errno == EINTR));
+	if (read_ret < 0)
 	{
 		if (errno == EAGAIN)
 			return 0;	/* no data there */
@@ -861,15 +893,19 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 
 	/* read urb header; %n argument may increment return value, but it's
 	* not mandatory, so does not count on it*/
-	string[ret] = 0;
+	string[read_ret] = 0;
 	ret = sscanf(string, "%x %d %c %c%c:%d:%d %s%n", &tag, &timestamp, &etype,
 		&pipeid1, &pipeid2, &dev_addr, &ep_num, status,
 		&cnt);
 	if (ret < 8)
 	{
-		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
-		    "Can't parse USB bus message '%s', too few tokens (expected 8 got %d)",
-		    string, ret);
+		char string_truncated[181];
+
+		strncpy(string_truncated, string, sizeof(string_truncated));
+		string_truncated[sizeof(string_truncated) - 1] = 0;
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+			 "Can't parse USB bus message '%s', too few tokens (expected 8 got %d)",
+			 string_truncated, ret);
 		return -1;
 	}
 	uhdr->id = tag;
@@ -886,7 +922,7 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 		return -1;
 	}
 	uhdr->ts_sec = pkth.ts.tv_sec;
-	uhdr->ts_usec = pkth.ts.tv_usec;
+	uhdr->ts_usec = (int32_t)pkth.ts.tv_usec;
 
 	/* parse endpoint information */
 	if (pipeid1 == 'C')
@@ -932,7 +968,7 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 		str5, &cnt);
 		if (ret < 5)
 		{
-			pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+			snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 				"Can't parse USB bus message '%s', too few tokens (expected 5 got %d)",
 				string, ret);
 			return -1;
@@ -956,7 +992,7 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 	ret = sscanf(string, " %d%n", &urb_len, &cnt);
 	if (ret < 1)
 	{
-		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		  "Can't parse urb length from '%s'", string);
 		return -1;
 	}
@@ -974,7 +1010,7 @@ usb_read_linux(pcap_t *handle, int max_packets _U_, pcap_handler callback, u_cha
 	/* check for data presence; data is present if and only if urb tag is '=' */
 	if (sscanf(string, " %c", &urb_tag) != 1)
 	{
-		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 			"Can't parse urb tag from '%s'", string);
 		return -1;
 	}
@@ -1010,7 +1046,7 @@ got:
 		pkth.caplen = (bpf_u_int32)handle->snapshot;
 
 	if (handle->fcode.bf_insns == NULL ||
-	    bpf_filter(handle->fcode.bf_insns, handle->buffer,
+	    pcap_filter(handle->fcode.bf_insns, handle->buffer,
 	      pkth.len, pkth.caplen)) {
 		handlep->packets_read++;
 		callback(user, &pkth, handle->buffer);
@@ -1020,9 +1056,9 @@ got:
 }
 
 static int
-usb_inject_linux(pcap_t *handle, const void *buf _U_, size_t size _U_)
+usb_inject_linux(pcap_t *handle, const void *buf _U_, int size _U_)
 {
-	pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+	snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 	    "Packet injection is not supported on USB devices");
 	return (-1);
 }
@@ -1031,13 +1067,14 @@ static int
 usb_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 {
 	struct pcap_usb_linux *handlep = handle->priv;
-	int dummy, ret, consumed, cnt;
+	int dummy, cnt;
+	ssize_t ret, consumed;
 	char string[USB_LINE_LEN];
 	char token[USB_LINE_LEN];
 	char * ptr = string;
 	int fd;
 
-	pcap_snprintf(string, USB_LINE_LEN, USB_TEXT_DIR"/%ds", handlep->bus_index);
+	snprintf(string, USB_LINE_LEN, USB_TEXT_DIR"/%ds", handlep->bus_index);
 	fd = open(string, O_RDONLY, 0);
 	if (fd < 0)
 	{
@@ -1047,7 +1084,7 @@ usb_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 			 * Not found at the new location; try the old
 			 * location.
 			 */
-			pcap_snprintf(string, USB_LINE_LEN, USB_TEXT_DIR_OLD"/%ds", handlep->bus_index);
+			snprintf(string, USB_LINE_LEN, USB_TEXT_DIR_OLD"/%ds", handlep->bus_index);
 			fd = open(string, O_RDONLY, 0);
 		}
 		if (fd < 0) {
@@ -1066,7 +1103,7 @@ usb_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 
 	if (ret < 0)
 	{
-		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 			"Can't read stats from fd %d ", fd);
 		return -1;
 	}
@@ -1108,6 +1145,10 @@ usb_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 static int
 usb_setdirection_linux(pcap_t *p, pcap_direction_t d)
 {
+	/*
+	 * It's guaranteed, at this point, that d is a valid
+	 * direction value.
+	 */
 	p->direction = d;
 	return 0;
 }
@@ -1208,11 +1249,11 @@ usb_read_linux_bin(pcap_t *handle, int max_packets _U_, pcap_handler callback, u
 		 */
 		pkth.len = sizeof(pcap_usb_header) + info.hdr->urb_len;
 	}
-	pkth.ts.tv_sec = info.hdr->ts_sec;
+	pkth.ts.tv_sec = (time_t)info.hdr->ts_sec;
 	pkth.ts.tv_usec = info.hdr->ts_usec;
 
 	if (handle->fcode.bf_insns == NULL ||
-	    bpf_filter(handle->fcode.bf_insns, handle->buffer,
+	    pcap_filter(handle->fcode.bf_insns, handle->buffer,
 	      pkth.len, pkth.caplen)) {
 		handlep->packets_read++;
 		callback(user, &pkth, handle->buffer);
@@ -1323,11 +1364,11 @@ usb_read_linux_mmap(pcap_t *handle, int max_packets, pcap_handler callback, u_ch
 				pkth.len = sizeof(pcap_usb_header_mmapped) +
 				    (hdr->ndesc * sizeof (usb_isodesc)) + hdr->urb_len;
 			}
-			pkth.ts.tv_sec = hdr->ts_sec;
+			pkth.ts.tv_sec = (time_t)hdr->ts_sec;
 			pkth.ts.tv_usec = hdr->ts_usec;
 
 			if (handle->fcode.bf_insns == NULL ||
-			    bpf_filter(handle->fcode.bf_insns, (u_char*) hdr,
+			    pcap_filter(handle->fcode.bf_insns, (u_char*) hdr,
 			      pkth.len, pkth.caplen)) {
 				handlep->packets_read++;
 				callback(user, &pkth, (u_char*) hdr);
@@ -1335,7 +1376,7 @@ usb_read_linux_mmap(pcap_t *handle, int max_packets, pcap_handler callback, u_ch
 			}
 		}
 
-		/* with max_packets specifying "unlimited" we stop afer the first chunk*/
+		/* with max_packets specifying "unlimited" we stop after the first chunk*/
 		if (PACKET_COUNT_IS_UNLIMITED(max_packets) || (packets == max_packets))
 			break;
 	}
